/*
Copyright (c) 2003-2011, CKSource - Frederico Knabben. All rights reserved.
For licensing, see LICENSE.html or http://ckeditor.com/license
*/

CKEDITOR.dialog.add( 'symbol', function( editor )
{
    var characterBeforeDiacritial = "&nbsp;";

	/**
	 * Simulate "this" of a dialog for non-dialog events.
	 * @type {CKEDITOR.dialog}
	 */
	var symbolDialog,
		lang = editor.lang.specialChar;

    function setCharTable(rangeDefinition, container, columns) {
        //Test if rangeDefinition is defined
        if (rangeDefinition != null) {
//            console.log("Setting table values...");
//            console.log(rangeDefinition);

            var charsTableLabel =  CKEDITOR.tools.getNextId() + '_symbol_table_label';
            var html = [''];
            
            var size = 0;
            var columnPos = 0;
            var initialized = false;
            var start, end, insertSpace;

            //Split the rangeDefinition
            var ranges = rangeDefinition.split(',');
            for (var k = 0; k < ranges.length; k++) {
                start = 0;
                end = -1;
                insertSpace = false; //Asume we are not inserting Combining characters
                var range = ranges[k].split('-');
                switch (range.length) {
                    case 0: {
                        //Error - do nothing
                        break;
                    }
                    case 1: {
                        //Only one character to insert
                        if (range[0].charAt(range[0].length - 1) == '*') {
                            //Range contains Combining characters
                            start = parseInt(range[0].substring(0, range[0].length - 1), 16);
                            end = start;
                            insertSpace = true;
                        } else {
                            start = parseInt(range[0], 16);
                            end = start;
                        }
                        break;
                    }
                    default: {
                        //Check start range
                        if (range[0].charAt(range[0].length - 1) == '*') {
                            //Range contains Combining characters
                            start = parseInt(range[0].substring(0, range[0].length - 1), 16);
                            insertSpace = true;
                        } else {
                            start = parseInt(range[0], 16);
                        }
                        //Check end range
                        if (range[1].charAt(range[1].length - 1) == '*') {
                            //Range contains Combining characters
                            end = parseInt(range[1].substring(0, range[1].length - 1), 16);
                            insertSpace = true;
                        } else {
                            end = parseInt(range[1], 16);
                        }
                        break;
                    }
                }

                //Sum up the complete range of characters to insert into the dialog
                size += end - start + 1;

                if (end >= start) {
                    //Only initialize once...
                    if (!initialized) {
                        html.push( '<table role="listbox" aria-labelledby="' + charsTableLabel + '"' +
                            ' style="width: 100%; height: 100%; border-collapse: separate;"' +
                            ' align="center" cellspacing="2" cellpadding="2" border="0">' );
                        initialized = true;
                    }

                    var i = start;
                    while ( i <= end ) {
                        if (columnPos == 0) {
                            //Start on a new row in the table
                            html.push('<tr>');
                        }

                        var endTableRow = true; //Asume when finished with the following loop that the table row should be finished

                        //Continue inserting columns fromk columnPos
                        for ( var j = columnPos ; j < columns ; j++, i++ ) {
                            if (i <= end) {
                                var character = getUnicode(i);//"&#" + i + ";";

                                // Use character in case description unavailable.
                                var characterDescription = lang[ i ] || character;

                                var charLabelId =  'cke_symbol_label_' + i + '_' + CKEDITOR.tools.getNextNumber();

                                html.push(
                                    '<td class="cke_dark_background" style="cursor: default" role="presentation">' +
                                    '<a href="javascript: void(0);" role="option"' +
                                    ' aria-posinset="' + ( i + 1 ) + '"',
                                    ' aria-setsize="' + size + '"',
                                    ' aria-labelledby="' + charLabelId + '"',
                                    ' style="cursor: inherit; display: block; height: 1.25em; margin-top: 0.25em; text-align: center;" title="', CKEDITOR.tools.htmlEncode( characterDescription ), '"' +
                                    ' onkeydown="CKEDITOR.tools.callFunction( ' + onKeydown + ', event, this )"' +
                                    ' onclick="CKEDITOR.tools.callFunction(' + onClick + ', this); return false;"' +
                                    ' tabindex="-1">' +
                                        '<span style="margin: 0 auto;cursor: inherit"' + (insertSpace ? '>' + characterBeforeDiacritial : '>') + character + '</span>' +
                                        '<span class="cke_voice_label" id="' + charLabelId + '">' + characterDescription + '</span>' +
                                    '</a>');

                                html.push( '</td>' );
                            } else {
                                //If we are at the end of the current range then jump out of the loop
                                columnPos = j;
                                endTableRow = false;
                                break;
                            }
                        }
                        //If row is finished then end it with </tr>
                        if (endTableRow) {
                            columnPos = 0;
                            html.push( '</tr>' );
                        } else {
                            //Current range is finished - jump on to next range
                            break;
                        }
                    }
                }
            }

            //Fill the rest of the columns with &nbsp; if neccessary
            if (columnPos != 0) {
                for (var i = columnPos; i < columns; i++) {
                    html.push('<td>&nbsp;</td>');
                }
                html.push( '</tr>' );
            }

            if (!initialized) {
                //If never initialized then write error message
                container.getElement().setHtml( '<div>No valid range(s) defined...</div>' );
            } else {
                if (size > 255) {
                    //Make scrollbars if the range is greater than 255 characters
                    html[0] = '<div style="width:100%;height:300px;overflow:auto;">';
                } else {
                    //No scrollbar needed
                    html[0] = '<div>';
                }

                html.push( '</table></div>' ) ;
                var sum = html.join( '' );
                container.getElement().setHtml( sum );
            }
        } else {
            //Error in rangeDefinition?!?
            container.getElement().setHtml( '<div>No range defined...</div>' );
        }
    }

    function getUnicode(numX) {
        var num = numX.toString(16).toUpperCase();
        if (num.length < 1) {
            return eval('"\\u0000"');
        } else if (num.length < 2) {
            return eval('"\\u000' + num+'"');
        } else if (num.length < 3) {
            return eval('"\\u00' + num+'"');
        } else if (num.length < 4) {
            return eval('"\\u0' + num+'"');
        } else {
            return eval('"\\u' + num+'"');
        }
    }

	var onChoice = function( evt )
	{
		var target, value;
		if ( evt.data )
			target = evt.data.getTarget();
		else
			target = new CKEDITOR.dom.element( evt );

		if ( target.getName() == 'a' && ( value = target.getChild( 0 ).getHtml() ) )
		{
			target.removeClass( "cke_light_background" );

            //Check if current character is a combining character
            if (value.length > characterBeforeDiacritial.length &&
                    value.indexOf(characterBeforeDiacritial) == 0) {
                //Remove extra character(s) before diacritical characters
                value = value.substring(characterBeforeDiacritial.length);
            }
			symbolDialog.hide();

//			editor.insertHtml( value );
            // We must use "insertText" here to keep text styled.
            var span = editor.document.createElement( 'span' );
            span.setHtml( value );
            editor.insertText( span.getText() );
		}
	};

	var onClick = CKEDITOR.tools.addFunction( onChoice );

	var focusedNode;

	var onFocus = function( evt, target )
	{
		var value;
		target = target || evt.data.getTarget();

		if ( target.getName() == 'span' )
			target = target.getParent();

		if ( target.getName() == 'a' && ( value = target.getChild( 0 ).getHtml() ) )
		{
			// Trigger blur manually if there is focused node.
			if ( focusedNode )
				onBlur( null, focusedNode );

			var unicodePreview = symbolDialog.getContentElement( 'info', 'unicodePreview' ).getElement();

			symbolDialog.getContentElement( 'info', 'charPreview' ).getElement().setHtml( value );
            var charCode = 0;
            if (value == '&nbsp;') {
                //This is a little tricky - when inserting a &#160; it is converted to a &nbsp; by Chrome automatically
                //so therefore the charCode is set to 160
                charCode = 160;
            } else if (value.length > characterBeforeDiacritial.length &&
                    value.indexOf(characterBeforeDiacritial) == 0) {
                //If a combining character is focused then write the charCode of the combining character and not the &nbsp; ;-)
                charCode = value.charCodeAt(characterBeforeDiacritial.length);
            } else {
                charCode = value.charCodeAt(0);
            }
            unicodePreview.setHtml( "Unicode (hex): " + charCode + " (" + charCode.toString(16).toUpperCase() + ")" );
			target.getParent().addClass( "cke_light_background" );

			// Memorize focused node.
			focusedNode = target;
		}
	};

	var onBlur = function( evt, target )
	{
		target = target || evt.data.getTarget();

		if ( target.getName() == 'span' )
			target = target.getParent();

		if ( target.getName() == 'a' )
		{
            symbolDialog.getContentElement( 'info', 'unicodePreview' ).getElement().setHtml( '&nbsp;' );
			symbolDialog.getContentElement( 'info', 'charPreview' ).getElement().setHtml( '&nbsp;' );
			target.getParent().removeClass( "cke_light_background" );

			focusedNode = undefined;
		}
	};

	var onKeydown = CKEDITOR.tools.addFunction( function( ev )
	{
		ev = new CKEDITOR.dom.event( ev );

		// Get an Anchor element.
		var element = ev.getTarget();
		var relative, nodeToMove;
		var keystroke = ev.getKeystroke(),
			rtl = editor.lang.dir == 'rtl';

		switch ( keystroke )
		{
			// UP-ARROW
			case 38 :
				// relative is TR
				if ( ( relative = element.getParent().getParent().getPrevious() ) )
				{
					nodeToMove = relative.getChild( [element.getParent().getIndex(), 0] );
					nodeToMove.focus();
					onBlur( null, element );
					onFocus( null, nodeToMove );
				}
				ev.preventDefault();
				break;
			// DOWN-ARROW
			case 40 :
				// relative is TR
				if ( ( relative = element.getParent().getParent().getNext() ) )
				{
					nodeToMove = relative.getChild( [ element.getParent().getIndex(), 0 ] );
					if ( nodeToMove && nodeToMove.type == 1 )
					{
						nodeToMove.focus();
						onBlur( null, element );
						onFocus( null, nodeToMove );
					}
				}
				ev.preventDefault();
				break;
			// SPACE
			// ENTER is already handled as onClick
			case 32 :
				onChoice( { data: ev } );
				ev.preventDefault();
				break;

			// RIGHT-ARROW
			case rtl ? 37 : 39 :
			// TAB
			case 9 :
				// relative is TD
				if ( ( relative = element.getParent().getNext() ) )
				{
					nodeToMove = relative.getChild( 0 );
					if ( nodeToMove.type == 1 )
					{
						nodeToMove.focus();
						onBlur( null, element );
						onFocus( null, nodeToMove );
						ev.preventDefault( true );
					}
					else
						onBlur( null, element );
				}
				// relative is TR
				else if ( ( relative = element.getParent().getParent().getNext() ) )
				{
					nodeToMove = relative.getChild( [ 0, 0 ] );
					if ( nodeToMove && nodeToMove.type == 1 )
					{
						nodeToMove.focus();
						onBlur( null, element );
						onFocus( null, nodeToMove );
						ev.preventDefault( true );
					}
					else
						onBlur( null, element );
				}
				break;

			// LEFT-ARROW
			case rtl ? 39 : 37 :
			// SHIFT + TAB
			case CKEDITOR.SHIFT + 9 :
				// relative is TD
				if ( ( relative = element.getParent().getPrevious() ) )
				{
					nodeToMove = relative.getChild( 0 );
					nodeToMove.focus();
					onBlur( null, element );
					onFocus( null, nodeToMove );
					ev.preventDefault( true );
				}
				// relative is TR
				else if ( ( relative = element.getParent().getParent().getPrevious() ) )
				{
					nodeToMove = relative.getLast().getChild( 0 );
					nodeToMove.focus();
					onBlur( null, element );
					onFocus( null, nodeToMove );
					ev.preventDefault( true );
				}
				else
					onBlur( null, element );
				break;
			default :
				// Do not stop not handled events.
				return;
		}
	});

    return {
        title : lang.title,
        buttons : [ CKEDITOR.dialog.cancelButton ],
        charColumns : 16,
        onShow : function( data )
        {
            var rangeElement = this.getContentElement( 'info', 'symbolRange' );
            var domElement = rangeElement.getInputElement().$;
            //Select first index if possible
            if (domElement.childNodes.length > 0) {
                domElement.selectedIndex = 0;

                var charContainer = this.getContentElement( 'info', 'charContainer' );
                setCharTable(CKEDITOR.config.symbolRanges[0][1], charContainer, this.definition.charColumns);
            }
        },
		contents : [
			{
				id : 'info',
				label : editor.lang.common.generalTab,
				title : editor.lang.common.generalTab,
				padding : 0,
				align : 'top',
                onHide: function () {
                    if (symbolDialog) {
                        symbolDialog.destroy();
                    }
                }, 
				elements : [
					{
						type : 'vbox',
						align : 'top',
                        width: '100%',
						children :
						[
                            {
                                type : 'select',
                                id : 'symbolRange',
                                width: '100%',
                                items : CKEDITOR.config.symbolRanges,
                                size : 1,
                                onChange : function( api ) {
                                    var charContainer = symbolDialog.getContentElement( 'info', 'charContainer' );
                                    setCharTable(api.sender.getValue(), charContainer, symbolDialog.definition.charColumns);
                                }
                            },
							{
								type : 'html',
								id : 'charContainer',
								html : '',
								onMouseover : onFocus,
								onMouseout : onBlur,
								focus : function()
								{
									var firstChar = this.getElement().getElementsByTag( 'a' ).getItem( 0 );
									setTimeout( function()
									{
										firstChar.focus();
										onFocus( null, firstChar );
									}, 0 );
								},
								onShow : function()
								{
                                    var firstChar = this.getElement().getElementsByTag( 'a' ).getItem( 0 );
                                    if (firstChar != null) {
                                        setTimeout( function()
                                            {
                                                firstChar.focus();
                                                onFocus( null, firstChar );
                                            }, 0 );
                                    }
								},
								onLoad : function( event )
								{
									symbolDialog = event.sender;
								}
							},
							{
								type : 'hbox',
								align : 'top',
								widths : [ '100%' ],
								children :
								[
                                    {
                                        type : 'html',
                                        html : '<div></div>'
                                    },
                                    {
                                        type : 'html',
                                        id : 'unicodePreview',
                                        className : 'cke_dark_background',
                                        style : 'border:1px solid #eeeeee;font-size:14px;height:20px;width:192px;padding-top:25px;font-family:\'Microsoft Sans Serif\',Arial,\'Arial Unicode\',Helvetica,Verdana;text-align:center;',
                                        html : '<div>&nbsp;</div>'
                                    },
                                    {
                                        type : 'html',
                                        id : 'charPreview',
                                        className : 'cke_dark_background',
                                        style : 'border:1px solid #eeeeee;font-size:28px;height:40px;width:70px;padding-top:9px;font-family:\'Microsoft Sans Serif\',Arial,\'Arial Unicode\',Helvetica,Verdana;text-align:center;',
                                        html : '<div>&nbsp;</div>'
                                    }
								]
							}
						]
					}
				]
			}
		]
    };
} );
